from typing import Optional

from .base_api import BaseAPI
from .models import (
    Measure,
    Person,
    Measures,
    Persons,
    Members,
    Member,
    Tutors,
    Tutor
)


class VirtualRoom(BaseAPI):
    def __init__(
            self,
            base_link: str,
            secret_key: str,
            app_id: str
    ):
        super().__init__(
            base_link,
            secret_key,
            app_id
        )

    async def get_persons(
            self,
            limit: int = 200,
            offset: int = 0
    ) -> Optional[Persons]:
        """
        Получение информации о физических лицах
        :rtype: list[Person]
        :param limit: Количество записей (200 максимально)
        :param offset: Сдвиг страницы
        :return: Sequence of Persons and count
        """
        persons = await self._get(
            route="/service/v2/persons",
            params={
                "limit": limit,
                "offset": offset,
            }
        )
        if persons:
            return Persons(
                [Person(**person) for person in persons['data']],
                persons['count']
            )
        else:
            return None

    async def get_person(self, person_id: int) -> Optional[Person]:
        """
        Получение информации о физическом лице
        :rtype: Person
        :param person_id: идентификатор физического лица
        :return: Person
        """
        person = await self._get(
            route=f"/service/v2/persons/{person_id}"
        )
        if person:
            return Person(**person)
        else:
            return None

    async def get_measures(
            self,
            limit: int = 200,
            offset: int = 0
    ) -> Optional[Measures]:
        """
        Получение информации о мероприятиях
        :param limit: Количество записей (200 максимально)
        :param offset: Сдвиг страницы
        :return: Sequence of Measures and count
        """
        measures = await self._get(
            route="/service/v2/measures",
            params={
                "limit": limit,
                "offset": offset,
            }
        )
        if measures:
            return Measures(
                [Measure(**measure) for measure in measures['data']],
                measures['count']
            )
        else:
            return None

    async def get_measure(self, measure_id: int) -> Optional[Measure]:
        """
        Получение информации о мероприятии
        :rtype: Measure
        :param measure_id: идентификатор мероприятия
        :return: Measure
        """
        measure = await self._get(
            route=f"/service/v2/measures/{measure_id}"
        )
        if measure:
            return Measure(**measure)
        else:
            return None

    async def get_members(
            self,
            measure_id: int,
            limit: int = 200,
            offset: int = 0
    ) -> Optional[Members]:
        """
        Получение информации об участниках мероприятия
        :param measure_id: идентификатор мероприятия
        :param limit: Количество записей (200 максимально)
        :param offset: Сдвиг страницы
        :return: Sequence of Members and count
        """
        members = await self._get(
            route=f"/service/v2/measures/{measure_id}/members",
            params={
                "limit": limit,
                "offset": offset,
            }
        )
        if members:
            return Members(
                [Member(**member) for member in members['data']],
                members['count']
            )
        else:
            return None

    async def get_tutors(
            self,
            measure_id: int,
            limit: int = 200,
            offset: int = 0
    ) -> Optional[Tutors]:
        """
        Получение информации о преподавателях мероприятия
        :param measure_id: идентификатор мероприятия
        :param limit: Количество записей (200 максимально)
        :param offset: Сдвиг страницы
        :return: Sequence of tutors and count
        """
        tutors = await self._get(
            route=f"/service/v2/measures/{measure_id}/tutors",
            params={
                "limit": limit,
                "offset": offset,
            }
        )
        if tutors:
            return Tutors(
                [Tutor(**tutor) for tutor in tutors['data']],
                tutors['count']
            )
        else:
            return None

    async def get_measures_info(self):
        measures_info = await self._get(
            route="/service/v2/measures/info"
        )
        return measures_info

    async def create_measure(
            self,
            measure: Measure,
    ) -> Optional[Measure]:
        """
        Добавление мероприятия
        :param measure: объект мероприятия
        :return: созданное мероприятие
        """
        created_measure = await self._post(
            route="/service/v2/measures",
            data=measure.model_dump(exclude_none=True),
        )
        if created_measure:
            return Measure(**created_measure)

    async def add_tutor_to_measure_by_email(
            self,
            measure_id: int,
            tutor_email: str,
            send_notifications: bool = True,
            add_roles_by_default: bool = True,
            enable_search_by_email: bool = True,
    ):
        """
        Добавление преподавателя мероприятия по E-mail
        :param measure_id: идентификатор мероприятия
        :param tutor_email: e-mail преподавателя
        :return: id физического лица
        """
        data = {
            "sendNotifications": send_notifications,
            "addRolesByDefault": add_roles_by_default,
            "enableSearchByEmail": enable_search_by_email,
        }
        tutor = await self._post(
            route=f"/service/v2/measures/{
                measure_id}/tutors/regbyemail/{tutor_email}",
            data=data
        )
        return tutor

    async def add_tutor_to_measure_by_id(
        self,
        measure_id: int,
        person_id: int,
    ) -> Optional[Tutor]:
        """
        Добавление преподавателя мероприятия по id
        :param measure_id: идентификатор мероприятия
        :param person_id: id физического лицапреподавателя
        :return: Tutor
        """
        tutor = await self._post(
            route=f"/service/v2/measures/{
                measure_id}/tutors/{person_id}",
        )
        if tutor:
            return Tutor(**tutor)

    async def delete_measure(self, measure_id: int) -> Optional[bool]:
        """
        Удаление мероприятия
        :param measure_id: идентификатор мероприятия
        :return: True if success
        """
        measure = await self._delete(
            route=f"/service/v2/measures/{measure_id}"
        )
        if measure:
            return True
        else:
            return False



import json
import logging
from typing import Optional
import aiohttp
from hashlib import md5
import urllib.parse as urlparse
from urllib.parse import urlencode


class BaseAPI:
    def __init__(
            self,
            base_link: str,
            secret_key: str,
            app_id: str,
    ):
        self._link = base_link

        self._secret_key = secret_key
        self._app_id = app_id

        self._base_link = base_link
        self._base_params = {
            "appid": self._app_id,
            "secretkey": self._secret_key,
        }

        self._headers = {
            "Accept": "*/*",
        }

    async def _prepare_params(self, route: str, params: Optional[dict] = None):
        if params is None:
            params = {}

        url_parts = list(urlparse.urlparse(self._base_link + route))
        query = dict(urlparse.parse_qsl(url_parts[4]))

        params.update(self._base_params)
        query.update(params)

        url_parts[4] = urlencode(query)

        legacy_url = urlparse.urlunparse(url_parts)
        md5_hash = md5(legacy_url.encode())
        sign = md5_hash.hexdigest().upper()

        encrypted_params = {
            "appid": self._app_id,
            "sign": sign
        }
        params.update(encrypted_params)
        params.pop("secretkey")
        return params

    async def _get(self, route: str, params: Optional[dict] = None):
        """
        Send get request to host
        :param params: request params
        :param route: request link
        :return: json object from host
        """
        params = await self._prepare_params(route, params)
        logging.info(f"GET {self._link}{route} {params=}")
        try:
            async with aiohttp.ClientSession(headers=self._headers) as session:
                async with session.get(
                    url=f"{self._link}{route}",
                    params=params,
                    verify_ssl=False,
                ) as get:
                    if get.ok:
                        answer = await get.json()
                        logging.info(f"GET {get.status} {
                            self._link}{route} {answer=}")
                        if 'Content-Range' in get.headers:
                            count = int(
                                get.headers['Content-Range'].split("/")[-1])
                            return {'data': answer, 'count': count}
                        else:
                            return answer
                    else:
                        logging.warning(f"GET {get.status} {
                            self._link}{route}")
                        error = await get.json()
                        raise aiohttp.ClientConnectionError(error)
        except aiohttp.ClientConnectionError as e:
            logging.warning(f"Api error: {self._link}{route} {e}")
        except Exception as e:
            logging.warning(f"Api is unreachable: {e}")

    async def _post(
            self,
            route: str,
            params: Optional[dict] = None,
            data: Optional[dict] = None,
    ) -> Optional[dict | str]:
        """
        Send post request to host
        :param params: request params
        :param data: request data
        :param route: request link
        :return: json object from host
        """
        params = await self._prepare_params(route, params)
        logging.info(f"POST {self._link}{route} {params=} {data=}")
        headers = {"Content-Type": "application/json"}
        try:
            async with aiohttp.ClientSession(headers=headers) as session:
                async with session.post(
                    f'{self._link}{route}',
                    params=params,
                    data=json.dumps(data, indent=4),
                    verify_ssl=False,
                ) as post:
                    if post.ok:
                        if post.content_type == "text/plain":
                            answer = await post.text()
                            logging.info(f"POST {post.status} {
                                self._link}{route} {answer=}")
                            return answer
                        answer = await post.json()
                        logging.info(f"POST {post.status} {
                            self._link}{route} {answer=}")
                        return answer
                    else:
                        logging.warning(f"POST {post.status} {
                            self._link}{route}")
                        error = await post.json()
                        raise aiohttp.ClientConnectionError(error)
        except aiohttp.ClientConnectionError as e:
            logging.warning(f"Api error: {self._link}{route} {e}")
        except Exception as e:
            logging.warning(f"Api is unreachable: {e}")

    async def _delete(
            self,
            route: str,
            params: Optional[dict] = None
    ) -> Optional[int]:
        params = await self._prepare_params(route, params)
        logging.info(f"DELETE {self._link}{route} {params=}")
        try:
            async with aiohttp.ClientSession(headers=self._headers) as session:
                async with session.delete(
                    url=f"{self._link}{route}",
                    params=params,
                    verify_ssl=False,
                ) as delete:
                    if delete.ok:
                        logging.info(f"DELETE {delete.status} {
                            self._link}{route}")
                        return delete.status
                    else:
                        logging.warning(f"DELETE {delete.status} {
                            self._link}{route}")
                        error = await delete.json()
                        raise aiohttp.ClientConnectionError(error)
        except aiohttp.ClientConnectionError as e:
            logging.warning(f"Api error: {self._link}{route} {e}")
        except Exception as e:
            logging.warning(f"Api is unreachable: {e}")

    # async def get_data(self, route: str, params: Optional[dict] = None):
    #     if params is None:
    #         params = {}
    #     params.update(self.make_params(route))
    #     logging.info(f"GET DATA {self._link}{route} with {params=}")
    #     try:
    #         async with aiohttp.ClientSession(headers=self.headers) as session:
    #             async with session.get(
    #                 url=f"{self._link}{route}",
    #                 params=params,
    #                 verify_ssl=False,
    #             ) as resp:
    #                 logging.info(f"{resp=}")
    #                 if resp.ok:
    #                     logging.info(f"{resp.status=} {self._link}{route}")
    #                     return await resp.read()
    #                 else:
    #                     raise aiohttp.ClientError
    #     except aiohttp.ClientConnectionError:
    #         logging.warning(f"Api is unreachable {self._link}{route}")
    #     except Exception as e:
    #         logging.warning(f"Api is unreachable: {e}")
    #
    # async def put(self, route: str, data: Optional[dict] = None) -> aiohttp.ClientResponse:
    #     """
    #     Send put request to host
    #     :param route: request link
    #     :param data: json object to send
    #     :return: json object from host
    #     """
    #     headers = self.headers
    #     headers.update({"Content-Type": "application/x-www-form-urlencoded"})
    #     if data is None:
    #         data = {}
    #     logging.info(f"Sending PUT request to {self._link}{route} {data=}")
    #     try:
    #         async with aiohttp.ClientSession(headers=headers) as session:
    #             async with session.put(
    #                 f'{self._link}{route}',
    #                 data=data,
    #                 verify_ssl=False,
    #             ) as put:
    #                 logging.info(f"{put=}")
    #                 if put.ok:
    #                     logging.info(f"{put.status=} {self._link}{route} {data=}")
    #                     return put
    #                 else:
    #                     raise aiohttp.ClientError
    #     except aiohttp.ClientConnectionError:
    #         logging.warning(f"Api is unreachable {self._link}{route}")
    #     except Exception as e:
    #         logging.warning(f"Api is unreachable: {e}")
    #
